import sys
import datetime as dt
from typing import Optional, Union, Sequence, Tuple, Any, overload, TypeVar

from numpy import (
    ndarray,
    number,
    integer,
    bool_,
    generic,
    _OrderKACF,
    _OrderACF,
    _ArrayLikeBool,
    _ArrayLikeIntOrBool,
    _ModeKind,
    _PartitionKind,
    _SortKind,
    _SortSide,
)
from numpy.typing import (
    DTypeLike,
    ArrayLike,
    _ShapeLike,
    _Shape,
    _IntLike,
    _BoolLike,
    _NumberLike,
)

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

# Various annotations for scalars

# While dt.datetime and dt.timedelta are not technically part of NumPy,
# they are one of the rare few builtin scalars which serve as valid return types.
# See https://github.com/numpy/numpy-stubs/pull/67#discussion_r412604113.
_ScalarNumpy = Union[generic, dt.datetime, dt.timedelta]
_ScalarBuiltin = Union[str, bytes, dt.date, dt.timedelta, bool, int, float, complex]
_Scalar = Union[_ScalarBuiltin, _ScalarNumpy]

# Integers and booleans can generally be used interchangeably
_ScalarIntOrBool = TypeVar("_ScalarIntOrBool", bound=Union[integer, bool_])
_ScalarGeneric = TypeVar("_ScalarGeneric", bound=generic)
_ScalarGenericDT = TypeVar(
    "_ScalarGenericDT", bound=Union[dt.datetime, dt.timedelta, generic]
)

_Number = TypeVar("_Number", bound=number)

# The signature of take() follows a common theme with its overloads:
# 1. A generic comes in; the same generic comes out
# 2. A scalar comes in; a generic comes out
# 3. An array-like object comes in; some keyword ensures that a generic comes out
# 4. An array-like object comes in; an ndarray or generic comes out
@overload
def take(
    a: _ScalarGenericDT,
    indices: int,
    axis: Optional[int] = ...,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> _ScalarGenericDT: ...
@overload
def take(
    a: _Scalar,
    indices: int,
    axis: Optional[int] = ...,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> _ScalarNumpy: ...
@overload
def take(
    a: ArrayLike,
    indices: int,
    axis: Optional[int] = ...,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> _ScalarNumpy: ...
@overload
def take(
    a: ArrayLike,
    indices: _ArrayLikeIntOrBool,
    axis: Optional[int] = ...,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> Union[_ScalarNumpy, ndarray]: ...
def reshape(a: ArrayLike, newshape: _ShapeLike, order: _OrderACF = ...) -> ndarray: ...
@overload
def choose(
    a: _ScalarIntOrBool,
    choices: ArrayLike,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> _ScalarIntOrBool: ...
@overload
def choose(
    a: Union[_IntLike, _BoolLike], choices: ArrayLike, out: Optional[ndarray] = ..., mode: _ModeKind = ...
) -> Union[integer, bool_]: ...
@overload
def choose(
    a: _ArrayLikeIntOrBool,
    choices: ArrayLike,
    out: Optional[ndarray] = ...,
    mode: _ModeKind = ...,
) -> ndarray: ...
def repeat(
    a: ArrayLike, repeats: _ArrayLikeIntOrBool, axis: Optional[int] = ...
) -> ndarray: ...
def put(
    a: ndarray, ind: _ArrayLikeIntOrBool, v: ArrayLike, mode: _ModeKind = ...
) -> None: ...
def swapaxes(a: ArrayLike, axis1: int, axis2: int) -> ndarray: ...
def transpose(
    a: ArrayLike, axes: Union[None, Sequence[int], ndarray] = ...
) -> ndarray: ...
def partition(
    a: ArrayLike,
    kth: _ArrayLikeIntOrBool,
    axis: Optional[int] = ...,
    kind: _PartitionKind = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> ndarray: ...
@overload
def argpartition(
    a: generic,
    kth: _ArrayLikeIntOrBool,
    axis: Optional[int] = ...,
    kind: _PartitionKind = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> integer: ...
@overload
def argpartition(
    a: _ScalarBuiltin,
    kth: _ArrayLikeIntOrBool,
    axis: Optional[int] = ...,
    kind: _PartitionKind = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> ndarray: ...
@overload
def argpartition(
    a: ArrayLike,
    kth: _ArrayLikeIntOrBool,
    axis: Optional[int] = ...,
    kind: _PartitionKind = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> ndarray: ...
def sort(
    a: ArrayLike,
    axis: Optional[int] = ...,
    kind: Optional[_SortKind] = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> ndarray: ...
def argsort(
    a: ArrayLike,
    axis: Optional[int] = ...,
    kind: Optional[_SortKind] = ...,
    order: Union[None, str, Sequence[str]] = ...,
) -> ndarray: ...
@overload
def argmax(a: ArrayLike, axis: None = ..., out: Optional[ndarray] = ...) -> integer: ...
@overload
def argmax(
    a: ArrayLike, axis: int = ..., out: Optional[ndarray] = ...
) -> Union[integer, ndarray]: ...
@overload
def argmin(a: ArrayLike, axis: None = ..., out: Optional[ndarray] = ...) -> integer: ...
@overload
def argmin(
    a: ArrayLike, axis: int = ..., out: Optional[ndarray] = ...
) -> Union[integer, ndarray]: ...
@overload
def searchsorted(
    a: ArrayLike,
    v: _Scalar,
    side: _SortSide = ...,
    sorter: Optional[_ArrayLikeIntOrBool] = ...,  # 1D int array
) -> integer: ...
@overload
def searchsorted(
    a: ArrayLike,
    v: ArrayLike,
    side: _SortSide = ...,
    sorter: Optional[_ArrayLikeIntOrBool] = ...,  # 1D int array
) -> ndarray: ...
def resize(a: ArrayLike, new_shape: _ShapeLike) -> ndarray: ...
@overload
def squeeze(a: _ScalarGeneric, axis: Optional[_ShapeLike] = ...) -> _ScalarGeneric: ...
@overload
def squeeze(a: ArrayLike, axis: Optional[_ShapeLike] = ...) -> ndarray: ...
def diagonal(
    a: ArrayLike, offset: int = ..., axis1: int = ..., axis2: int = ...  # >= 2D array
) -> ndarray: ...
def trace(
    a: ArrayLike,  # >= 2D array
    offset: int = ...,
    axis1: int = ...,
    axis2: int = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
) -> Union[number, ndarray]: ...
def ravel(a: ArrayLike, order: _OrderKACF = ...) -> ndarray: ...
def nonzero(a: ArrayLike) -> Tuple[ndarray, ...]: ...
def shape(a: ArrayLike) -> _Shape: ...
def compress(
    condition: ArrayLike,  # 1D bool array
    a: ArrayLike,
    axis: Optional[int] = ...,
    out: Optional[ndarray] = ...,
) -> ndarray: ...
@overload
def clip(
    a: _Number,
    a_min: ArrayLike,
    a_max: Optional[ArrayLike],
    out: Optional[ndarray] = ...,
    **kwargs: Any,
) -> _Number: ...
@overload
def clip(
    a: _Number,
    a_min: None,
    a_max: ArrayLike,
    out: Optional[ndarray] = ...,
    **kwargs: Any,
) -> _Number: ...
@overload
def clip(
    a: ArrayLike,
    a_min: ArrayLike,
    a_max: Optional[ArrayLike],
    out: Optional[ndarray] = ...,
    **kwargs: Any,
) -> Union[number, ndarray]: ...
@overload
def clip(
    a: ArrayLike,
    a_min: None,
    a_max: ArrayLike,
    out: Optional[ndarray] = ...,
    **kwargs: Any,
) -> Union[number, ndarray]: ...
@overload
def sum(
    a: _Number,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> _Number: ...
@overload
def sum(
    a: ArrayLike,
    axis: _ShapeLike = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> Union[number, ndarray]: ...
@overload
def all(
    a: ArrayLike,
    axis: None = ...,
    out: Optional[ndarray] = ...,
    keepdims: Literal[False] = ...,
) -> bool_: ...
@overload
def all(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
) -> Union[bool_, ndarray]: ...
@overload
def any(
    a: ArrayLike,
    axis: None = ...,
    out: Optional[ndarray] = ...,
    keepdims: Literal[False] = ...,
) -> bool_: ...
@overload
def any(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
) -> Union[bool_, ndarray]: ...
def cumsum(
    a: ArrayLike,
    axis: Optional[int] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
) -> ndarray: ...
@overload
def ptp(
    a: _Number,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
) -> _Number: ...
@overload
def ptp(
    a: ArrayLike,
    axis: None = ...,
    out: Optional[ndarray] = ...,
    keepdims: Literal[False] = ...,
) -> number: ...
@overload
def ptp(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
) -> Union[number, ndarray]: ...
@overload
def amax(
    a: _Number,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> _Number: ...
@overload
def amax(
    a: ArrayLike,
    axis: None = ...,
    out: Optional[ndarray] = ...,
    keepdims: Literal[False] = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> number: ...
@overload
def amax(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> Union[number, ndarray]: ...
@overload
def amin(
    a: _Number,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> _Number: ...
@overload
def amin(
    a: ArrayLike,
    axis: None = ...,
    out: Optional[ndarray] = ...,
    keepdims: Literal[False] = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> number: ...
@overload
def amin(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> Union[number, ndarray]: ...

# TODO: `np.prod()``: For object arrays `initial` does not necessarily
# have to be a numerical scalar.
# The only requirement is that it is compatible
# with the `.__mul__()` method(s) of the passed array's elements.

# Note that the same situation holds for all wrappers around
# `np.ufunc.reduce`, e.g. `np.sum()` (`.__add__()`).
@overload
def prod(
    a: _Number,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: None = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> _Number: ...
@overload
def prod(
    a: ArrayLike,
    axis: None = ...,
    dtype: DTypeLike = ...,
    out: None = ...,
    keepdims: Literal[False] = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> number: ...
@overload
def prod(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
    initial: _NumberLike = ...,
    where: _ArrayLikeBool = ...,
) -> Union[number, ndarray]: ...
def cumprod(
    a: ArrayLike,
    axis: Optional[int] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
) -> ndarray: ...
def ndim(a: ArrayLike) -> int: ...
def size(a: ArrayLike, axis: Optional[int] = ...) -> int: ...
@overload
def around(
    a: _Number, decimals: int = ..., out: Optional[ndarray] = ...
) -> _Number: ...
@overload
def around(
    a: _NumberLike, decimals: int = ..., out: Optional[ndarray] = ...
) -> number: ...
@overload
def around(
    a: ArrayLike, decimals: int = ..., out: Optional[ndarray] = ...
) -> ndarray: ...
@overload
def mean(
    a: ArrayLike,
    axis: None = ...,
    dtype: DTypeLike = ...,
    out: None = ...,
    keepdims: Literal[False] = ...,
) -> number: ...
@overload
def mean(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    keepdims: bool = ...,
) -> Union[number, ndarray]: ...
@overload
def std(
    a: ArrayLike,
    axis: None = ...,
    dtype: DTypeLike = ...,
    out: None = ...,
    ddof: int = ...,
    keepdims: Literal[False] = ...,
) -> number: ...
@overload
def std(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    ddof: int = ...,
    keepdims: bool = ...,
) -> Union[number, ndarray]: ...
@overload
def var(
    a: ArrayLike,
    axis: None = ...,
    dtype: DTypeLike = ...,
    out: None = ...,
    ddof: int = ...,
    keepdims: Literal[False] = ...,
) -> number: ...
@overload
def var(
    a: ArrayLike,
    axis: Optional[_ShapeLike] = ...,
    dtype: DTypeLike = ...,
    out: Optional[ndarray] = ...,
    ddof: int = ...,
    keepdims: bool = ...,
) -> Union[number, ndarray]: ...



import numpy as np

np.dtype(dtype=np.int64)
np.dtype(int)
np.dtype("int")
np.dtype(None)

np.dtype((int, 2))
np.dtype((int, (1,)))

np.dtype({"names": ["a", "b"], "formats": [int, float]})
np.dtype({"names": ["a"], "formats": [int], "titles": [object]})
np.dtype({"names": ["a"], "formats": [int], "titles": [object()]})

np.dtype([("name", np.unicode_, 16), ("grades", np.float64, (2,)), ("age", "int32")])

np.dtype(
    {
        "names": ["a", "b"],
        "formats": [int, float],
        "itemsize": 9,
        "aligned": False,
        "titles": ["x", "y"],
        "offsets": [0, 1],
    }
)

np.dtype((np.float_, float))


class Test:
    dtype = float


np.dtype(Test())



import os
import pytest

import numpy as np
from numpy.testing import assert_, assert_raises, assert_equal, assert_string_equal

from . import util


def _path(*a):
    return os.path.join(*((os.path.dirname(__file__),) + a))


class TestIntentInOut(util.F2PyTest):
    # Check that intent(in out) translates as intent(inout)
    sources = [_path('src', 'regression', 'inout.f90')]

    @pytest.mark.slow
    def test_inout(self):
        # non-contiguous should raise error
        x = np.arange(6, dtype=np.float32)[::2]
        assert_raises(ValueError, self.module.foo, x)

        # check values with contiguous array
        x = np.arange(3, dtype=np.float32)
        self.module.foo(x)
        assert_equal(x, [3, 1, 2])
 

class TestNumpyVersionAttribute(util.F2PyTest):
    # Check that th attribute __f2py_numpy_version__ is present
    # in the compiled module and that has the value np.__version__.
    sources = [_path('src', 'regression', 'inout.f90')]
    
    @pytest.mark.slow
    def test_numpy_version_attribute(self):
        
        # Check that self.module has an attribute named "__f2py_numpy_version__"
        assert_(hasattr(self.module, "__f2py_numpy_version__"), 
                msg="Fortran module does not have __f2py_numpy_version__")
        
        # Check that the attribute __f2py_numpy_version__ is a string
        assert_(isinstance(self.module.__f2py_numpy_version__, str),
                msg="__f2py_numpy_version__ is not a string")
        
        # Check that __f2py_numpy_version__ has the value numpy.__version__
        assert_string_equal(np.__version__, self.module.__f2py_numpy_version__)



import sys
from typing import (
    Any,
    Optional,
    Union,
    Sequence,
    Tuple,
    Callable,
    List,
    overload,
    TypeVar,
    Iterable,
)

from numpy import ndarray, generic, dtype, bool_, signedinteger, _OrderKACF, _OrderCF
from numpy.typing import ArrayLike, DTypeLike, _ShapeLike

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

_T = TypeVar("_T")
_ArrayType = TypeVar("_ArrayType", bound=ndarray)

_CorrelateMode = Literal["valid", "same", "full"]

@overload
def zeros_like(
    a: _ArrayType,
    dtype: None = ...,
    order: _OrderKACF = ...,
    subok: Literal[True] = ...,
    shape: None = ...,
) -> _ArrayType: ...
@overload
def zeros_like(
    a: ArrayLike,
    dtype: DTypeLike = ...,
    order: _OrderKACF = ...,
    subok: bool = ...,
    shape: Optional[_ShapeLike] = ...,
) -> ndarray: ...
def ones(
    shape: _ShapeLike,
    dtype: DTypeLike = ...,
    order: _OrderCF = ...,
    *,
    like: ArrayLike = ...,
) -> ndarray: ...
@overload
def ones_like(
    a: _ArrayType,
    dtype: None = ...,
    order: _OrderKACF = ...,
    subok: Literal[True] = ...,
    shape: None = ...,
) -> _ArrayType: ...
@overload
def ones_like(
    a: ArrayLike,
    dtype: DTypeLike = ...,
    order: _OrderKACF = ...,
    subok: bool = ...,
    shape: Optional[_ShapeLike] = ...,
) -> ndarray: ...
@overload
def empty_like(
    a: _ArrayType,
    dtype: None = ...,
    order: _OrderKACF = ...,
    subok: Literal[True] = ...,
    shape: None = ...,
) -> _ArrayType: ...
@overload
def empty_like(
    a: ArrayLike,
    dtype: DTypeLike = ...,
    order: _OrderKACF = ...,
    subok: bool = ...,
    shape: Optional[_ShapeLike] = ...,
) -> ndarray: ...
def full(
    shape: _ShapeLike,
    fill_value: Any,
    dtype: DTypeLike = ...,
    order: _OrderCF = ...,
    *,
    like: ArrayLike = ...,
) -> ndarray: ...
@overload
def full_like(
    a: _ArrayType,
    fill_value: Any,
    dtype: None = ...,
    order: _OrderKACF = ...,
    subok: Literal[True] = ...,
    shape: None = ...,
) -> _ArrayType: ...
@overload
def full_like(
    a: ArrayLike,
    fill_value: Any,
    dtype: DTypeLike = ...,
    order: _OrderKACF = ...,
    subok: bool = ...,
    shape: Optional[_ShapeLike] = ...,
) -> ndarray: ...
@overload
def count_nonzero(
    a: ArrayLike, axis: None = ..., *, keepdims: Literal[False] = ...
) -> int: ...
@overload
def count_nonzero(
    a: ArrayLike, axis: _ShapeLike = ..., *, keepdims: bool = ...
) -> Union[signedinteger[Any], ndarray]: ...  # TODO: np.intp
def isfortran(a: Union[ndarray, generic]) -> bool: ...
def argwhere(a: ArrayLike) -> ndarray: ...
def flatnonzero(a: ArrayLike) -> ndarray: ...
def correlate(a: ArrayLike, v: ArrayLike, mode: _CorrelateMode = ...) -> ndarray: ...
def convolve(a: ArrayLike, v: ArrayLike, mode: _CorrelateMode = ...) -> ndarray: ...
@overload
def outer(a: ArrayLike, b: ArrayLike, out: None = ...) -> ndarray: ...
@overload
def outer(a: ArrayLike, b: ArrayLike, out: _ArrayType = ...) -> _ArrayType: ...
def tensordot(
    a: ArrayLike,
    b: ArrayLike,
    axes: Union[int, Tuple[_ShapeLike, _ShapeLike]] = ...,
) -> ndarray: ...
def roll(
    a: ArrayLike,
    shift: _ShapeLike,
    axis: Optional[_ShapeLike] = ...,
) -> ndarray: ...
def rollaxis(a: ndarray, axis: int, start: int = ...) -> ndarray: ...
def moveaxis(
    a: ndarray,
    source: _ShapeLike,
    destination: _ShapeLike,
) -> ndarray: ...
def cross(
    a: ArrayLike,
    b: ArrayLike,
    axisa: int = ...,
    axisb: int = ...,
    axisc: int = ...,
    axis: Optional[int] = ...,
) -> ndarray: ...
@overload
def indices(
    dimensions: Sequence[int],
    dtype: DTypeLike = ...,
    sparse: Literal[False] = ...,
) -> ndarray: ...
@overload
def indices(
    dimensions: Sequence[int],
    dtype: DTypeLike = ...,
    sparse: Literal[True] = ...,
) -> Tuple[ndarray, ...]: ...
def fromfunction(
    function: Callable[..., _T],
    shape: Sequence[int],
    *,
    dtype: DTypeLike = ...,
    like: ArrayLike = ...,
    **kwargs: Any,
) -> _T: ...
def isscalar(element: Any) -> bool: ...
def binary_repr(num: int, width: Optional[int] = ...) -> str: ...
def base_repr(number: int, base: int = ..., padding: int = ...) -> str: ...
def identity(n: int, dtype: DTypeLike = ..., *, like: ArrayLike = ...) -> ndarray: ...
def allclose(
    a: ArrayLike,
    b: ArrayLike,
    rtol: float = ...,
    atol: float = ...,
    equal_nan: bool = ...,
) -> bool: ...
def isclose(
    a: ArrayLike,
    b: ArrayLike,
    rtol: float = ...,
    atol: float = ...,
    equal_nan: bool = ...,
) -> Union[bool_, ndarray]: ...
def array_equal(a1: ArrayLike, a2: ArrayLike) -> bool: ...
def array_equiv(a1: ArrayLike, a2: ArrayLike) -> bool: ...



"""
Tests for miscellaneous (non-magic) ``np.ndarray``/``np.generic`` methods.

More extensive tests are performed for the methods'
function-based counterpart in `../from_numeric.py`.

"""

import numpy as np

class SubClass(np.ndarray): ...

f8: np.float64
A: np.ndarray
B: SubClass

reveal_type(f8.all())  # E: numpy.bool_
reveal_type(A.all())  # E: numpy.bool_
reveal_type(A.all(axis=0))  # E: Union[numpy.bool_, numpy.ndarray]
reveal_type(A.all(keepdims=True))  # E: Union[numpy.bool_, numpy.ndarray]
reveal_type(A.all(out=B))  # E: SubClass

reveal_type(f8.any())  # E: numpy.bool_
reveal_type(A.any())  # E: numpy.bool_
reveal_type(A.any(axis=0))  # E: Union[numpy.bool_, numpy.ndarray]
reveal_type(A.any(keepdims=True))  # E: Union[numpy.bool_, numpy.ndarray]
reveal_type(A.any(out=B))  # E: SubClass

reveal_type(f8.argmax())  # E: numpy.signedinteger[Any]
reveal_type(A.argmax())  # E: numpy.signedinteger[Any]
reveal_type(A.argmax(axis=0))  # E: Union[numpy.signedinteger[Any], numpy.ndarray]
reveal_type(A.argmax(out=B))  # E: SubClass

reveal_type(f8.argmin())  # E: numpy.signedinteger[Any]
reveal_type(A.argmin())  # E: numpy.signedinteger[Any]
reveal_type(A.argmin(axis=0))  # E: Union[numpy.signedinteger[Any], numpy.ndarray]
reveal_type(A.argmin(out=B))  # E: SubClass

reveal_type(f8.argsort())  # E: numpy.ndarray
reveal_type(A.argsort())  # E: numpy.ndarray

reveal_type(f8.astype(np.int64).choose([()]))  # E: numpy.ndarray
reveal_type(A.choose([0]))  # E: numpy.ndarray
reveal_type(A.choose([0], out=B))  # E: SubClass

reveal_type(f8.clip(1))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.clip(1))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.clip(None, 1))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.clip(1, out=B))  # E: SubClass
reveal_type(A.clip(None, 1, out=B))  # E: SubClass

reveal_type(f8.compress([0]))  # E: numpy.ndarray
reveal_type(A.compress([0]))  # E: numpy.ndarray
reveal_type(A.compress([0], out=B))  # E: SubClass

reveal_type(f8.conj())  # E: numpy.floating[numpy.typing._64Bit]
reveal_type(A.conj())  # E: numpy.ndarray
reveal_type(B.conj())  # E: SubClass

reveal_type(f8.conjugate())  # E: numpy.floating[numpy.typing._64Bit]
reveal_type(A.conjugate())  # E: numpy.ndarray
reveal_type(B.conjugate())  # E: SubClass

reveal_type(f8.cumprod())  # E: numpy.ndarray
reveal_type(A.cumprod())  # E: numpy.ndarray
reveal_type(A.cumprod(out=B))  # E: SubClass

reveal_type(f8.cumsum())  # E: numpy.ndarray
reveal_type(A.cumsum())  # E: numpy.ndarray
reveal_type(A.cumsum(out=B))  # E: SubClass

reveal_type(f8.max())  # E: numpy.number[Any]
reveal_type(A.max())  # E: numpy.number[Any]
reveal_type(A.max(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.max(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.max(out=B))  # E: SubClass

reveal_type(f8.mean())  # E: numpy.number[Any]
reveal_type(A.mean())  # E: numpy.number[Any]
reveal_type(A.mean(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.mean(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.mean(out=B))  # E: SubClass

reveal_type(f8.min())  # E: numpy.number[Any]
reveal_type(A.min())  # E: numpy.number[Any]
reveal_type(A.min(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.min(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.min(out=B))  # E: SubClass

reveal_type(f8.newbyteorder())  # E: numpy.floating[numpy.typing._64Bit]
reveal_type(A.newbyteorder())  # E: numpy.ndarray
reveal_type(B.newbyteorder('|'))  # E: SubClass

reveal_type(f8.prod())  # E: numpy.number[Any]
reveal_type(A.prod())  # E: numpy.number[Any]
reveal_type(A.prod(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.prod(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.prod(out=B))  # E: SubClass

reveal_type(f8.ptp())  # E: numpy.number[Any]
reveal_type(A.ptp())  # E: numpy.number[Any]
reveal_type(A.ptp(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.ptp(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.ptp(out=B))  # E: SubClass

reveal_type(f8.round())  # E: numpy.floating[numpy.typing._64Bit]
reveal_type(A.round())  # E: numpy.ndarray
reveal_type(A.round(out=B))  # E: SubClass

reveal_type(f8.repeat(1))  # E: numpy.ndarray
reveal_type(A.repeat(1))  # E: numpy.ndarray
reveal_type(B.repeat(1))  # E: numpy.ndarray

reveal_type(f8.std())  # E: numpy.number[Any]
reveal_type(A.std())  # E: numpy.number[Any]
reveal_type(A.std(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.std(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.std(out=B))  # E: SubClass

reveal_type(f8.sum())  # E: numpy.number[Any]
reveal_type(A.sum())  # E: numpy.number[Any]
reveal_type(A.sum(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.sum(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.sum(out=B))  # E: SubClass

reveal_type(f8.take(0))  # E: numpy.generic
reveal_type(A.take(0))  # E: numpy.generic
reveal_type(A.take([0]))  # E: numpy.ndarray
reveal_type(A.take(0, out=B))  # E: SubClass
reveal_type(A.take([0], out=B))  # E: SubClass

reveal_type(f8.var())  # E: numpy.number[Any]
reveal_type(A.var())  # E: numpy.number[Any]
reveal_type(A.var(axis=0))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.var(keepdims=True))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.var(out=B))  # E: SubClass

reveal_type(A.argpartition([0]))  # E: numpy.ndarray

reveal_type(A.diagonal())  # E: numpy.ndarray

reveal_type(A.dot(1))  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.dot(1, out=B))  # E: SubClass

reveal_type(A.nonzero())  # E: tuple[numpy.ndarray]

reveal_type(A.searchsorted([1]))  # E: numpy.ndarray

reveal_type(A.trace())  # E: Union[numpy.number[Any], numpy.ndarray]
reveal_type(A.trace(out=B))  # E: SubClass



import sys
from typing import TypeVar, overload, List, Sequence

from numpy import ndarray
from numpy.typing import ArrayLike

if sys.version_info >= (3, 8):
    from typing import SupportsIndex
else:
    from typing_extensions import Protocol
    class SupportsIndex(Protocol):
        def __index__(self) -> int: ...

_ArrayType = TypeVar("_ArrayType", bound=ndarray)

@overload
def atleast_1d(__arys: ArrayLike) -> ndarray: ...
@overload
def atleast_1d(*arys: ArrayLike) -> List[ndarray]: ...

@overload
def atleast_2d(__arys: ArrayLike) -> ndarray: ...
@overload
def atleast_2d(*arys: ArrayLike) -> List[ndarray]: ...

@overload
def atleast_3d(__arys: ArrayLike) -> ndarray: ...
@overload
def atleast_3d(*arys: ArrayLike) -> List[ndarray]: ...

def vstack(tup: Sequence[ArrayLike]) -> ndarray: ...
def hstack(tup: Sequence[ArrayLike]) -> ndarray: ...
@overload
def stack(
    arrays: Sequence[ArrayLike], axis: SupportsIndex = ..., out: None = ...
) -> ndarray: ...
@overload
def stack(
    arrays: Sequence[ArrayLike], axis: SupportsIndex = ..., out: _ArrayType = ...
) -> _ArrayType: ...
def block(arrays: ArrayLike) -> ndarray: ...



"""Tests for :mod:`numpy.core.fromnumeric`."""

import numpy as np

A = np.array(True, ndmin=2, dtype=bool)
A.setflags(write=False)

a = np.bool_(True)

np.take(a, None)  # E: No overload variant of "take" matches argument type
np.take(a, axis=1.0)  # E: No overload variant of "take" matches argument type
np.take(A, out=1)  # E: No overload variant of "take" matches argument type
np.take(A, mode="bob")  # E: No overload variant of "take" matches argument type

np.reshape(a, None)  # E: Argument 2 to "reshape" has incompatible type
np.reshape(A, 1, order="bob")  # E: Argument "order" to "reshape" has incompatible type

np.choose(a, None)  # E: No overload variant of "choose" matches argument type
np.choose(a, out=1.0)  # E: No overload variant of "choose" matches argument type
np.choose(A, mode="bob")  # E: No overload variant of "choose" matches argument type

np.repeat(a, None)  # E: Argument 2 to "repeat" has incompatible type
np.repeat(A, 1, axis=1.0)  # E: Argument "axis" to "repeat" has incompatible type

np.swapaxes(A, None, 1)  # E: Argument 2 to "swapaxes" has incompatible type
np.swapaxes(A, 1, [0])  # E: Argument 3 to "swapaxes" has incompatible type

np.transpose(A, axes=1.0)  # E: Argument "axes" to "transpose" has incompatible type

np.partition(a, None)  # E: Argument 2 to "partition" has incompatible type
np.partition(
    a, 0, axis="bob"  # E: Argument "axis" to "partition" has incompatible type
)
np.partition(
    A, 0, kind="bob"  # E: Argument "kind" to "partition" has incompatible type
)
np.partition(
    A, 0, order=range(5)  # E: Argument "order" to "partition" has incompatible type
)

np.argpartition(  # E: No overload variant of "argpartition" matches argument type
    a, None
)
np.argpartition(  # E: No overload variant of "argpartition" matches argument type
    a, 0, axis="bob"
)
np.argpartition(  # E: No overload variant of "argpartition" matches argument type
    A, 0, kind="bob"
)
np.argpartition(
    A, 0, order=range(5)  # E: Argument "order" to "argpartition" has incompatible type
)

np.sort(A, axis="bob")  # E: Argument "axis" to "sort" has incompatible type
np.sort(A, kind="bob")  # E: Argument "kind" to "sort" has incompatible type
np.sort(A, order=range(5))  # E: Argument "order" to "sort" has incompatible type

np.argsort(A, axis="bob")  # E: Argument "axis" to "argsort" has incompatible type
np.argsort(A, kind="bob")  # E: Argument "kind" to "argsort" has incompatible type
np.argsort(A, order=range(5))  # E: Argument "order" to "argsort" has incompatible type

np.argmax(A, axis="bob")  # E: No overload variant of "argmax" matches argument type
np.argmax(A, kind="bob")  # E: No overload variant of "argmax" matches argument type

np.argmin(A, axis="bob")  # E: No overload variant of "argmin" matches argument type
np.argmin(A, kind="bob")  # E: No overload variant of "argmin" matches argument type

np.searchsorted(  # E: No overload variant of "searchsorted" matches argument type
    A[0], 0, side="bob"
)
np.searchsorted(  # E: No overload variant of "searchsorted" matches argument type
    A[0], 0, sorter=1.0
)

np.resize(A, 1.0)  # E: Argument 2 to "resize" has incompatible type

np.squeeze(A, 1.0)  # E: No overload variant of "squeeze" matches argument type

np.diagonal(A, offset=None)  # E: Argument "offset" to "diagonal" has incompatible type
np.diagonal(A, axis1="bob")  # E: Argument "axis1" to "diagonal" has incompatible type
np.diagonal(A, axis2=[])  # E: Argument "axis2" to "diagonal" has incompatible type

np.trace(A, offset=None)  # E: Argument "offset" to "trace" has incompatible type
np.trace(A, axis1="bob")  # E: Argument "axis1" to "trace" has incompatible type
np.trace(A, axis2=[])  # E: Argument "axis2" to "trace" has incompatible type

np.ravel(a, order="bob")  # E: Argument "order" to "ravel" has incompatible type

np.compress(
    [True], A, axis=1.0  # E: Argument "axis" to "compress" has incompatible type
)

np.clip(a, 1, 2, out=1)  # E: No overload variant of "clip" matches argument type
np.clip(1, None, None)  # E: No overload variant of "clip" matches argument type

np.sum(a, axis=1.0)  # E: No overload variant of "sum" matches argument type
np.sum(a, keepdims=1.0)  # E: No overload variant of "sum" matches argument type
np.sum(a, initial=[1])  # E: No overload variant of "sum" matches argument type

np.all(a, axis=1.0)  # E: No overload variant of "all" matches argument type
np.all(a, keepdims=1.0)  # E: No overload variant of "all" matches argument type
np.all(a, out=1.0)  # E: No overload variant of "all" matches argument type

np.any(a, axis=1.0)  # E: No overload variant of "any" matches argument type
np.any(a, keepdims=1.0)  # E: No overload variant of "any" matches argument type
np.any(a, out=1.0)  # E: No overload variant of "any" matches argument type

np.cumsum(a, axis=1.0)  # E: Argument "axis" to "cumsum" has incompatible type
np.cumsum(a, dtype=1.0)  # E: Argument "dtype" to "cumsum" has incompatible type
np.cumsum(a, out=1.0)  # E: Argument "out" to "cumsum" has incompatible type

np.ptp(a, axis=1.0)  # E: No overload variant of "ptp" matches argument type
np.ptp(a, keepdims=1.0)  # E: No overload variant of "ptp" matches argument type
np.ptp(a, out=1.0)  # E: No overload variant of "ptp" matches argument type

np.amax(a, axis=1.0)  # E: No overload variant of "amax" matches argument type
np.amax(a, keepdims=1.0)  # E: No overload variant of "amax" matches argument type
np.amax(a, out=1.0)  # E: No overload variant of "amax" matches argument type
np.amax(a, initial=[1.0])  # E: No overload variant of "amax" matches argument type
np.amax(a, where=[1.0])  # E: List item 0 has incompatible type

np.amin(a, axis=1.0)  # E: No overload variant of "amin" matches argument type
np.amin(a, keepdims=1.0)  # E: No overload variant of "amin" matches argument type
np.amin(a, out=1.0)  # E: No overload variant of "amin" matches argument type
np.amin(a, initial=[1.0])  # E: No overload variant of "amin" matches argument type
np.amin(a, where=[1.0])  # E: List item 0 has incompatible type

np.prod(a, axis=1.0)  # E: No overload variant of "prod" matches argument type
np.prod(a, out=False)  # E: No overload variant of "prod" matches argument type
np.prod(a, keepdims=1.0)  # E: No overload variant of "prod" matches argument type
np.prod(a, initial=int)  # E: No overload variant of "prod" matches argument type
np.prod(a, where=1.0)  # E: No overload variant of "prod" matches argument type

np.cumprod(a, axis=1.0)  # E: Argument "axis" to "cumprod" has incompatible type
np.cumprod(a, out=False)  # E: Argument "out" to "cumprod" has incompatible type

np.size(a, axis=1.0)  # E: Argument "axis" to "size" has incompatible type

np.around(a, decimals=1.0)  # E: No overload variant of "around" matches argument type
np.around(a, out=type)  # E: No overload variant of "around" matches argument type

np.mean(a, axis=1.0)  # E: No overload variant of "mean" matches argument type
np.mean(a, out=False)  # E: No overload variant of "mean" matches argument type
np.mean(a, keepdims=1.0)  # E: No overload variant of "mean" matches argument type

np.std(a, axis=1.0)  # E: No overload variant of "std" matches argument type
np.std(a, out=False)  # E: No overload variant of "std" matches argument type
np.std(a, ddof='test')  # E: No overload variant of "std" matches argument type
np.std(a, keepdims=1.0)  # E: No overload variant of "std" matches argument type

np.var(a, axis=1.0)  # E: No overload variant of "var" matches argument type
np.var(a, out=False)  # E: No overload variant of "var" matches argument type
np.var(a, ddof='test')  # E: No overload variant of "var" matches argument type
np.var(a, keepdims=1.0)  # E: No overload variant of "var" matches argument type
